@@ -3,9 +3,9 @@ Arithmetic - Do arithmetic operations on images.
 Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,10 +31,14 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/array.h>
+#include <gnuastro/binary.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 #include <gnuastro/statistics.h>
 #include <gnuastro/arithmetic.h>
+#include <gnuastro/interpolate.h>
 
 #include <gnuastro-internal/checkset.h>
 
@@ -131,14 +135,16 @@ pop_number_of_operands(struct arithmeticparams *p, gal_data_t *data,
 
 struct arithmetic_filter_p
 {
-  int      operator;            /* The type of filtering.          */
-  size_t     *fsize;            /* Filter size.                    */
-  size_t   *hpfsize;            /* Positive Half-filter size.      */
-  size_t   *hnfsize;            /* Negative Half-filter size.      */
-  gal_data_t *input;            /* Input dataset.                  */
-  gal_data_t   *out;            /* Output dataset.                 */
+  int           operator;       /* The type of filtering.                */
+  size_t          *fsize;       /* Filter size.                          */
+  size_t        *hpfsize;       /* Positive Half-filter size.            */
+  size_t        *hnfsize;       /* Negative Half-filter size.            */
+  float     sclip_multip;       /* Sigma multiple in sigma-clipping.     */
+  float      sclip_param;       /* Termination critera in sigma-cliping. */
+  gal_data_t      *input;       /* Input dataset.                        */
+  gal_data_t        *out;       /* Output dataset.                       */
 
-  int      hasblank;            /* If the dataset has blank values.*/
+  int           hasblank;       /* If the dataset has blank values.      */
 };
 
 
@@ -153,9 +159,9 @@ arithmetic_filter(void *in_prm)
   struct arithmetic_filter_p *afp=(struct arithmetic_filter_p *)tprm->params;
   gal_data_t *input=afp->input;
 
-  size_t ind,index;
-  int out_type_checked=0;
-  gal_data_t *result=NULL;
+  size_t sind=-1;
+  size_t ind, index, one=1;
+  gal_data_t *sigclip, *result=NULL;
   size_t *hpfsize=afp->hpfsize, *hnfsize=afp->hnfsize;
   size_t *tsize, *dsize=input->dsize, *fsize=afp->fsize;
   size_t i, j, coord[ARITHMETIC_FILTER_DIM], ndim=input->ndim;
@@ -172,8 +178,10 @@ arithmetic_filter(void *in_prm)
   /* Go over all the pixels that were assigned to this thread. */
   for(i=0; tprm->indexs[i] != GAL_BLANK_SIZE_T; ++i)
     {
-      /* Get the coordinate of the pixel. */
+      /* For easy reading, put the index in `ind'. */
       ind=tprm->indexs[i];
+
+      /* Get the coordinate of the pixel. */
       gal_dimension_index_to_coord(ind, ndim, dsize, coord);
 
       /* See which dimensions need trimming. */
@@ -196,7 +204,7 @@ arithmetic_filter(void *in_prm)
                            : coord[j] + hpfsize[j] + 1);
               tsize[j] = end[j] - start[j];
             }
-          else  /* We are NOT on the edge (given requested filter width). */
+          else  /* NOT on the edge (given requested filter width). */
             {
               tsize[j] = fsize[j];
               start[j] = coord[j] - hnfsize[j];
@@ -204,9 +212,15 @@ arithmetic_filter(void *in_prm)
           tile->size *= tsize[j];
         }
 
+      /* For a test.
+         printf("coord: %zu, %zu\n", coord[1]+1, coord[0]+1);
+         printf("\tstart: %zu, %zu\n", start[1]+1, start[0]+1);
+         printf("\ttsize: %zu, %zu\n", tsize[1], tsize[0]);
+      */
+
       /* Set the tile's starting pointer. */
       index=gal_dimension_coord_to_index(ndim, dsize, start);
-      tile->array=gal_data_ptr_increment(input->array, index, input->type);
+      tile->array=gal_pointer_increment(input->array, index, input->type);
 
       /* Do the necessary calculation. */
       switch(afp->operator)
@@ -215,42 +229,64 @@ arithmetic_filter(void *in_prm)
           result=gal_statistics_median(tile, 0);
           break;
 
+
         case ARITHMETIC_OP_FILTER_MEAN:
           result=gal_statistics_mean(tile);
           break;
 
+
+        case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
+          /* Find the sigma-clipped results. */
+          sigclip=gal_statistics_sigma_clip(tile, afp->sclip_multip,
+                                            afp->sclip_param, 0, 1);
+
+          /* Set the required index. */
+          switch(afp->operator)
+            {
+            case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:   sind = 2; break;
+            case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN: sind = 1; break;
+            default:
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                    "%s to fix the problem. The `afp->operator' value "
+                    "%d is not recognized as sigma-clipped median or "
+                    "mean", __func__, PACKAGE_BUGREPORT, afp->operator);
+            }
+
+          /* Allocate the output and write the value into it. */
+          result=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL,
+                                0, -1, NULL, NULL, NULL);
+          ((float *)(result->array))[0] =
+            ((float *)(sigclip->array))[sind];
+
+          /* Clean up. */
+          gal_data_free(sigclip);
+          break;
+
+
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
-                "fix the problem. `afp->operator' code %d is not "
-                "recognized", PACKAGE_BUGREPORT, __func__, afp->operator);
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
+                "to fix the problem. `afp->operator' code %d is not "
+                "recognized", PACKAGE_BUGREPORT, __func__,
+                afp->operator);
         }
 
-      /* Make sure the output array type and result's type are the same. We
-         only need to do this once, but we'll suffice to once for each
-         thread for simplicify of the code, it is too negligible to have
-         any real affect. */
-      if( out_type_checked==0)
-        {
-          if(result->type!=afp->out->type )
-            error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so "
-                  "we can address the problem. The tyes of `result' and "
-                  "`out' aren't the same, they are respectively: `%s' and "
-                  "`%s'", __func__, PACKAGE_BUGREPORT,
-                  gal_type_name(result->type, 1),
-                  gal_type_name(afp->out->type, 1));
-          out_type_checked=1;
-        }
+      /* Make sure the output array type and result's type are the
+         same. */
+      if(result->type!=afp->out->type)
+        result=gal_data_copy_to_new_type_free(result, afp->out->type);
+
 
       /* Copy the result into the output array. */
-      memcpy(gal_data_ptr_increment(afp->out->array, ind, afp->out->type),
+      memcpy(gal_pointer_increment(afp->out->array, ind, afp->out->type),
              result->array, gal_type_sizeof(afp->out->type));
 
-      /* Clean up. */
+      /* Clean up for this pixel. */
       gal_data_free(result);
     }
 
 
-  /* Clean up. */
+  /* Clean up for this thread. */
   tile->array=NULL;
   tile->block=NULL;
   gal_data_free(tile);
@@ -268,12 +304,14 @@ arithmetic_filter(void *in_prm)
 static void
 wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 {
-  size_t i=0, ndim, one=1;
   int type=GAL_TYPE_INVALID;
+  size_t i=0, ndim, nparams, one=1;
   struct arithmetic_filter_p afp={0};
   size_t fsize[ARITHMETIC_FILTER_DIM];
-  gal_data_t *tmp, *tmp2, *zero, *comp, *fsize_list=NULL;
+  gal_data_t *tmp, *tmp2, *zero, *comp, *params_list=NULL;
   size_t hnfsize[ARITHMETIC_FILTER_DIM], hpfsize[ARITHMETIC_FILTER_DIM];
+  int issigclip=(operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEAN
+                 || operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN);
 
 
   /* Get the input's number of dimensions. */
@@ -297,21 +335,40 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
                       NULL, NULL);
 
 
-  /* Based on the dimensions of the popped operand, pop the necessary
-     number of operands. */
-  for(i=0;i<ndim;++i)
-    gal_list_data_add(&fsize_list, operands_pop(p, token));
+  /* Based on the first popped operand's dimensions and the operator, of
+     pop the necessary number of operands. */
+  nparams = ndim + (issigclip ? 2 : 0 );
+  for(i=0;i<nparams;++i)
+    gal_list_data_add(&params_list, operands_pop(p, token));
 
 
-  /* Make sure the filter size only has single values. */
+  /* Make sure the parameters only have single values. */
   i=0;
-  for(tmp=fsize_list; tmp!=NULL; tmp=tmp->next)
+  for(tmp=params_list; tmp!=NULL; tmp=tmp->next)
     {
       ++i;
       if(tmp->size!=1)
-        error(EXIT_FAILURE, 0, "the filter length values given to the "
-              "filter operators can only be numbers. Value number %zu has "
-              "%zu elements, so its an array", ndim-i-1, tmp->size);
+        error(EXIT_FAILURE, 0, "the parameters given to the filtering "
+              "operators can only be numbers. Value number %zu has %zu "
+              "elements, so its an array", i, tmp->size);
+    }
+
+
+  /* If this is a sigma-clipping filter, the top two operands are the
+     sigma-clipping parameters. */
+  if(issigclip)
+    {
+      /* Read the sigma-clipping multiple (first element in the list). */
+      tmp=gal_list_data_pop(&params_list);
+      tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_FLOAT32);
+      afp.sclip_multip=*(float *)(tmp->array);
+      gal_data_free(tmp);
+
+      /* Read the sigma-clipping termination parameter. */
+      tmp=gal_list_data_pop(&params_list);
+      tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_FLOAT32);
+      afp.sclip_param=*(float *)(tmp->array);
+      gal_data_free(tmp);
     }
 
 
@@ -324,7 +381,7 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
          values must be written in the inverse order since the user gives
          dimensions with the FITS standard. */
       i=ndim-1;
-      for(tmp=fsize_list; tmp!=NULL; tmp=tmp->next)
+      for(tmp=params_list; tmp!=NULL; tmp=tmp->next)
         {
           /* Make sure the user has given an integer type. */
           if(tmp->type==GAL_TYPE_FLOAT32 || tmp->type==GAL_TYPE_FLOAT64)
@@ -338,6 +395,7 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
             error(EXIT_FAILURE, 0, "lengths of filter along dimensions "
                   "must be positive. The given length in dimension %zu"
                   "is either zero or negative", ndim-i);
+          gal_data_free(comp);
 
           /* Convert the input into size_t and put it into the array that
              keeps the filter size. */
@@ -370,18 +428,26 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
             { hnfsize[i]=fsize[i]/2; hpfsize[i]=fsize[i]/2-1; }
         }
 
+      /* For a test.
+      printf("fsize: %zu, %zu\n", fsize[0], fsize[1]);
+      printf("hnfsize: %zu, %zu\n", hnfsize[0], hnfsize[1]);
+      printf("hpfsize: %zu, %zu\n", hpfsize[0], hpfsize[1]);
+      */
 
       /* See if the input has blank pixels. */
       afp.hasblank=gal_blank_present(afp.input, 1);
 
+
       /* Set the type of the output dataset. */
       switch(operator)
         {
         case ARITHMETIC_OP_FILTER_MEDIAN:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
           type=afp.input->type;
           break;
 
         case ARITHMETIC_OP_FILTER_MEAN:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
           type=GAL_TYPE_FLOAT64;
           break;
 
@@ -391,12 +457,14 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
                 PACKAGE_BUGREPORT, __func__, operator);
         }
 
+
       /* Allocate the output dataset. Note that filtering doesn't change
          the units of the dataset. */
       afp.out=gal_data_alloc(NULL, type, ndim, afp.input->dsize,
                              afp.input->wcs, 0, afp.input->minmapsize,
                              NULL, afp.input->unit, NULL);
 
+
       /* Spin off threads for each pixel. */
       gal_threads_spin_off(arithmetic_filter, &afp, afp.input->size,
                            p->cp.numthreads);
@@ -406,10 +474,10 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
   /* Add the output to the top of the stack. */
   operands_add(p, NULL, afp.out);
 
-
   /* Clean up and add the output on top of the stack */
+  gal_data_free(zero);
   gal_data_free(afp.input);
-  gal_list_data_free(fsize_list);
+  gal_list_data_free(params_list);
 }
 
 
@@ -430,6 +498,293 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 
 
 
+
+/***************************************************************/
+/*************            Other functions          *************/
+/***************************************************************/
+static int
+arithmetic_binary_conn_sanity_checks(gal_data_t *in, gal_data_t *conn,
+                                     char *operator)
+{
+  int conn_int;
+
+  /* Do proper sanity checks on `conn'. */
+  if(conn->size!=1)
+    error(EXIT_FAILURE, 0, "the first popped operand to `%s' must be a "
+          "single number. However, it has %zu elements", operator,
+          conn->size);
+  if(conn->type==GAL_TYPE_FLOAT32 || conn->type==GAL_TYPE_FLOAT64)
+    error(EXIT_FAILURE, 0, "the first popped operand to `%s' is the "
+          "connectivity (a value between 1 and the number of dimensions) "
+          "therefore, it must NOT be a floating point", operator);
+
+  /* Convert the connectivity value to a 32-bit integer and read it in and
+     make sure it is not larger than the number of dimensions. */
+  conn=gal_data_copy_to_new_type_free(conn, GAL_TYPE_INT32);
+  conn_int = *((int32_t *)(conn->array));
+  if(conn_int>in->ndim)
+    error(EXIT_FAILURE, 0, "the first popped operand of `%s' (%d) is "
+          "larger than the number of dimensions in the second-popped "
+          "operand (%zu)", operator, conn_int, in->ndim);
+
+  /* Make sure the array has an unsigned 8-bit type. */
+  if(in->type!=GAL_TYPE_UINT8)
+    error(EXIT_FAILURE, 0, "the second popped operand of `%s' doesn't "
+          "have an 8-bit unsigned integer type. It must be a binary "
+          "dataset (only being equal to zero is checked). You can use "
+          "the `uint8' operator for type conversion", operator);
+
+  /* Clean up and return the integer value of `conn'. */
+  gal_data_free(conn);
+  return conn_int;
+}
+
+
+
+
+
+static void
+arithmetic_erode_dilate(struct arithmeticparams *p, char *token, int op)
+{
+  int conn_int;
+
+  /* Pop the two necessary operands. */
+  gal_data_t *conn = operands_pop(p, token);
+  gal_data_t *in   = operands_pop(p, token);
+
+  /* Do the sanity checks and  */
+  switch(op)
+    {
+    case ARITHMETIC_OP_ERODE:
+      conn_int=arithmetic_binary_conn_sanity_checks(in, conn, "erode");
+      gal_binary_erode(in, 1, conn_int, 1);
+      break;
+
+    case ARITHMETIC_OP_DILATE:
+      conn_int=arithmetic_binary_conn_sanity_checks(in, conn, "dilate");
+      gal_binary_dilate(in, 1, conn_int, 1);
+      break;
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+            "problem. The operator code %d not recognized", __func__,
+            PACKAGE_BUGREPORT, op);
+    }
+
+  /* Push the result onto the stack. */
+  operands_add(p, NULL, in);
+
+  /* Recall that`conn' was freed in the sanity check. */
+}
+
+
+
+
+
+static void
+arithmetic_connected_components(struct arithmeticparams *p, char *token)
+{
+  int conn_int;
+  gal_data_t *out=NULL;
+
+  /* Pop the two necessary operands. */
+  gal_data_t *conn = operands_pop(p, token);
+  gal_data_t *in   = operands_pop(p, token);
+
+  /* Basic sanity checks. */
+  conn_int=arithmetic_binary_conn_sanity_checks(in, conn,
+                                                "connected-components");
+
+  /* Do the connected components labeling. */
+  gal_binary_connected_components(in, &out, conn_int);
+
+  /* Push the result onto the stack. */
+  operands_add(p, NULL, out);
+
+  /* Clean up (`conn' was freed in the sanity check). */
+  gal_data_free(in);
+}
+
+
+
+
+
+static void
+arithmetic_invert(struct arithmeticparams *p, char *token)
+{
+  gal_data_t *in = operands_pop(p, token);
+
+  uint8_t *u8  = in->array, *u8f  = u8  + in->size;
+  uint8_t *u16 = in->array, *u16f = u16 + in->size;
+  uint8_t *u32 = in->array, *u32f = u32 + in->size;
+  uint8_t *u64 = in->array, *u64f = u64 + in->size;
+
+  /* Do the inversion based on type. */
+  switch(in->type)
+    {
+    case GAL_TYPE_UINT8:  do *u8  = UINT8_MAX-*u8;   while(++u8<u8f);   break;
+    case GAL_TYPE_UINT16: do *u16 = UINT16_MAX-*u16; while(++u16<u16f); break;
+    case GAL_TYPE_UINT32: do *u32 = UINT32_MAX-*u32; while(++u32<u32f); break;
+    case GAL_TYPE_UINT64: do *u64 = UINT64_MAX-*u64; while(++u64<u64f); break;
+    default:
+      error(EXIT_FAILURE, 0, "`invert' operand has %s type. `invert' can "
+            "only take unsigned integer types.\n\nYou can use any of the "
+            "`uint8', `uint16', `uint32', or `uint64' operators to chage "
+            "the type before calling `invert'",
+            gal_type_name(in->type, 1));
+    }
+
+  /* Push the result onto the stack. */
+  operands_add(p, NULL, in);
+}
+
+
+
+
+
+static void
+arithmetic_interpolate(struct arithmeticparams *p, char *token)
+{
+  int num_int;
+  gal_data_t *interpolated;
+
+  /* First pop the number of nearby neighbors.*/
+  gal_data_t *num = operands_pop(p, token);
+
+  /* Then pop the actual dataset to interpolate. */
+  gal_data_t *in = operands_pop(p, token);
+
+  /* Do proper sanity checks on `num'. */
+  if(num->size!=1)
+    error(EXIT_FAILURE, 0, "the first popped operand to "
+          "`interpolate-medianngb' must be a single number. However, "
+          "it has %zu elements", num->size);
+  if(num->type==GAL_TYPE_FLOAT32 || num->type==GAL_TYPE_FLOAT64)
+    error(EXIT_FAILURE, 0, "the first popped operand to "
+          "`interpolate-medianngb' is the number of nearby neighbors (a "
+          "counter, an integer). It must NOT be a floating point.\n\n"
+          "If its already an integer, but in a floating point container, "
+          "you can use the `int32' operator to convert it to a 32-bit "
+          "integer for example");
+
+  /* Convert the given number to a 32-bit integer and read it in. */
+  num=gal_data_copy_to_new_type_free(num, GAL_TYPE_INT32);
+  num_int = *((int32_t *)(num->array));
+
+  /* Call the interpolation function. */
+  interpolated=gal_interpolate_close_neighbors(in, NULL, num_int,
+                                               p->cp.numthreads, 1, 0);
+
+  /* Clean up and push the interpolated array onto the stack. */
+  gal_data_free(in);
+  gal_data_free(num);
+  operands_add(p, NULL, interpolated);
+}
+
+
+
+
+
+static void
+arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
+{
+  long dim;
+  gal_data_t *collapsed=NULL;
+
+  /* First popped operand is the dimension. */
+  gal_data_t *dimension = operands_pop(p, token);
+
+  /* The second popped operand is the desired input dataset. */
+  gal_data_t *input = operands_pop(p, token);
+
+
+  /* Small sanity check. */
+  if( dimension->ndim!=1 || dimension->size!=1)
+    error(EXIT_FAILURE, 0, "First popped operand of `collapse-*' operators "
+          "(dimension to collapse) must be a single number (single-element, "
+          "one-dimensional dataset). But it has %zu dimension(s) and %zu "
+          "element(s).", dimension->ndim, dimension->size);
+  if(dimension->type==GAL_TYPE_FLOAT32 || dimension->type==GAL_TYPE_FLOAT64)
+    error(EXIT_FAILURE, 0, "First popped operand of `collapse-*' operators "
+          "(dimension to collapse) must have an integer type, but it has "
+          "a floating point type (`%s')", gal_type_name(dimension->type,1));
+  dimension=gal_data_copy_to_new_type_free(dimension, GAL_TYPE_LONG);
+  dim=((long *)(dimension->array))[0];
+  if(dim<0 || dim==0)
+    error(EXIT_FAILURE, 0, "First popped operand of `collapse-*' operators "
+          "(dimension to collapse) must be positive (larger than zero), it "
+          "is %ld", dim);
+
+
+  /* If a WCS structure has been read, we'll need to pass it to
+     `gal_dimension_collapse', so it modifies it respectively. */
+  input->wcs=p->refdata.wcs;
+
+
+  /* Run the relevant library function. */
+  switch(operator)
+    {
+    case ARITHMETIC_OP_COLLAPSE_SUM:
+      collapsed=gal_dimension_collapse_sum(input, input->ndim-dim, NULL);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MEAN:
+      collapsed=gal_dimension_collapse_mean(input, input->ndim-dim, NULL);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_NUMBER:
+      collapsed=gal_dimension_collapse_number(input, input->ndim-dim);
+      break;
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+            "problem. The operator code %d is not recognized", __func__,
+            PACKAGE_BUGREPORT, operator);
+    }
+
+
+  /* If a WCS structure existed, a modified WCS is now present in
+     `collapsed->wcs'. So we'll let the freeing of `input' free the old
+     `p->refdata.wcs' structure and we'll put the new one there, then we'll
+     set `collapsed->wcs' to `NULL', so the new one isn't freed. */
+  p->refdata.wcs = collapsed->wcs;
+  collapsed->wcs = NULL;
+
+
+  /* We'll also need to correct the size of the reference dataset. We'll
+     use `memcpy' to write the new `dsize' values into the old ones. The
+     dimensions have decreased, so we won't be writing outside of allocated
+     space that `p->refdata.dsize' points to. */
+  p->refdata.ndim -= 1;
+  memcpy( p->refdata.dsize, collapsed->dsize,
+           p->refdata.ndim * (sizeof *p->refdata.dsize) );
+
+
+  /* Clean up and add the collapsed dataset to the top of the operands. */
+  gal_data_free(input);
+  gal_data_free(dimension);
+  operands_add(p, NULL, collapsed);
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 /***************************************************************/
 /*************      Reverse Polish algorithm       *************/
@@ -453,7 +808,7 @@ reversepolish(struct arithmeticparams *p)
 
   /* Prepare the processing: */
   p->operands=NULL;
-  p->addcounter=p->popcounter=0;
+  p->popcounter=0;
 
 
   /* Go over each input token and do the work. */
@@ -462,140 +817,164 @@ reversepolish(struct arithmeticparams *p)
       /* If we have a name or number, then add it to the operands linked
          list. Otherwise, pull out two members and do the specified
          operation on them. */
-      if(gal_fits_name_is_fits(token->v))
+      if( gal_array_name_recognized(token->v)
+          || operands_is_name(p, token->v) )
         operands_add(p, token->v, NULL);
       else if( (d1=gal_data_copy_string_to_number(token->v)) )
         operands_add(p, NULL, d1);
+      else if( !strncmp(token->v, SET_OPERATOR_PREFIX,
+                        SET_OPERATOR_PREFIX_LENGTH) )
+        operands_set_name(p, token->v);
       else
         {
 
           /* Order is the same as in the manual. */
           /* Simple arithmetic operators. */
           if      (!strcmp(token->v, "+" ))
-            { op=GAL_ARITHMETIC_OP_PLUS;          nop=2;  }
+            { op=GAL_ARITHMETIC_OP_PLUS;              nop=2;  }
           else if (!strcmp(token->v, "-" ))
-            { op=GAL_ARITHMETIC_OP_MINUS;         nop=2;  }
+            { op=GAL_ARITHMETIC_OP_MINUS;             nop=2;  }
           else if (!strcmp(token->v, "x" ))
-            { op=GAL_ARITHMETIC_OP_MULTIPLY;      nop=2;  }
+            { op=GAL_ARITHMETIC_OP_MULTIPLY;          nop=2;  }
           else if (!strcmp(token->v, "/" ))
-            { op=GAL_ARITHMETIC_OP_DIVIDE;        nop=2;  }
+            { op=GAL_ARITHMETIC_OP_DIVIDE;            nop=2;  }
           else if (!strcmp(token->v, "%" ))
-            { op=GAL_ARITHMETIC_OP_MODULO;        nop=2;  }
+            { op=GAL_ARITHMETIC_OP_MODULO;            nop=2;  }
 
           /* Mathematical Operators. */
           else if (!strcmp(token->v, "abs"))
-            { op=GAL_ARITHMETIC_OP_ABS;           nop=1;  }
+            { op=GAL_ARITHMETIC_OP_ABS;               nop=1;  }
           else if (!strcmp(token->v, "pow"))
-            { op=GAL_ARITHMETIC_OP_POW;           nop=2;  }
+            { op=GAL_ARITHMETIC_OP_POW;               nop=2;  }
           else if (!strcmp(token->v, "sqrt"))
-            { op=GAL_ARITHMETIC_OP_SQRT;          nop=1;  }
+            { op=GAL_ARITHMETIC_OP_SQRT;              nop=1;  }
           else if (!strcmp(token->v, "log"))
-            { op=GAL_ARITHMETIC_OP_LOG;           nop=1;  }
+            { op=GAL_ARITHMETIC_OP_LOG;               nop=1;  }
           else if (!strcmp(token->v, "log10"))
-            { op=GAL_ARITHMETIC_OP_LOG10;         nop=1;  }
+            { op=GAL_ARITHMETIC_OP_LOG10;             nop=1;  }
 
           /* Statistical/higher-level operators. */
           else if (!strcmp(token->v, "minvalue"))
-            { op=GAL_ARITHMETIC_OP_MINVAL;        nop=1;  }
+            { op=GAL_ARITHMETIC_OP_MINVAL;            nop=1;  }
           else if (!strcmp(token->v, "maxvalue"))
-            { op=GAL_ARITHMETIC_OP_MAXVAL;        nop=1;  }
+            { op=GAL_ARITHMETIC_OP_MAXVAL;            nop=1;  }
           else if (!strcmp(token->v, "numvalue"))
-            { op=GAL_ARITHMETIC_OP_NUMVAL;        nop=1;  }
+            { op=GAL_ARITHMETIC_OP_NUMVAL;            nop=1;  }
           else if (!strcmp(token->v, "sumvalue"))
-            { op=GAL_ARITHMETIC_OP_SUMVAL;        nop=1;  }
+            { op=GAL_ARITHMETIC_OP_SUMVAL;            nop=1;  }
           else if (!strcmp(token->v, "meanvalue"))
-            { op=GAL_ARITHMETIC_OP_MEANVAL;       nop=1;  }
+            { op=GAL_ARITHMETIC_OP_MEANVAL;           nop=1;  }
           else if (!strcmp(token->v, "stdvalue"))
-            { op=GAL_ARITHMETIC_OP_STDVAL;        nop=1;  }
+            { op=GAL_ARITHMETIC_OP_STDVAL;            nop=1;  }
           else if (!strcmp(token->v, "medianvalue"))
-            { op=GAL_ARITHMETIC_OP_MEDIANVAL;     nop=1;  }
+            { op=GAL_ARITHMETIC_OP_MEDIANVAL;         nop=1;  }
           else if (!strcmp(token->v, "min"))
-            { op=GAL_ARITHMETIC_OP_MIN;           nop=-1; }
+            { op=GAL_ARITHMETIC_OP_MIN;               nop=-1; }
           else if (!strcmp(token->v, "max"))
-            { op=GAL_ARITHMETIC_OP_MAX;           nop=-1; }
+            { op=GAL_ARITHMETIC_OP_MAX;               nop=-1; }
           else if (!strcmp(token->v, "num"))
-            { op=GAL_ARITHMETIC_OP_NUM;           nop=-1; }
+            { op=GAL_ARITHMETIC_OP_NUM;               nop=-1; }
           else if (!strcmp(token->v, "sum"))
-            { op=GAL_ARITHMETIC_OP_SUM;           nop=-1; }
+            { op=GAL_ARITHMETIC_OP_SUM;               nop=-1; }
           else if (!strcmp(token->v, "mean"))
-            { op=GAL_ARITHMETIC_OP_MEAN;          nop=-1; }
+            { op=GAL_ARITHMETIC_OP_MEAN;              nop=-1; }
           else if (!strcmp(token->v, "std"))
-            { op=GAL_ARITHMETIC_OP_STD;           nop=-1; }
+            { op=GAL_ARITHMETIC_OP_STD;               nop=-1; }
           else if (!strcmp(token->v, "median"))
-            { op=GAL_ARITHMETIC_OP_MEDIAN;        nop=-1; }
+            { op=GAL_ARITHMETIC_OP_MEDIAN;            nop=-1; }
 
           /* Conditional operators. */
           else if (!strcmp(token->v, "lt" ))
-            { op=GAL_ARITHMETIC_OP_LT;            nop=2;  }
+            { op=GAL_ARITHMETIC_OP_LT;                nop=2;  }
           else if (!strcmp(token->v, "le"))
-            { op=GAL_ARITHMETIC_OP_LE;            nop=2;  }
+            { op=GAL_ARITHMETIC_OP_LE;                nop=2;  }
           else if (!strcmp(token->v, "gt" ))
-            { op=GAL_ARITHMETIC_OP_GT;            nop=2;  }
+            { op=GAL_ARITHMETIC_OP_GT;                nop=2;  }
           else if (!strcmp(token->v, "ge"))
-            { op=GAL_ARITHMETIC_OP_GE;            nop=2;  }
+            { op=GAL_ARITHMETIC_OP_GE;                nop=2;  }
           else if (!strcmp(token->v, "eq"))
-            { op=GAL_ARITHMETIC_OP_EQ;            nop=2;  }
+            { op=GAL_ARITHMETIC_OP_EQ;                nop=2;  }
           else if (!strcmp(token->v, "ne"))
-            { op=GAL_ARITHMETIC_OP_NE;            nop=2;  }
+            { op=GAL_ARITHMETIC_OP_NE;                nop=2;  }
           else if (!strcmp(token->v, "and"))
-            { op=GAL_ARITHMETIC_OP_AND;           nop=2;  }
+            { op=GAL_ARITHMETIC_OP_AND;               nop=2;  }
           else if (!strcmp(token->v, "or"))
-            { op=GAL_ARITHMETIC_OP_OR;            nop=2;  }
+            { op=GAL_ARITHMETIC_OP_OR;                nop=2;  }
           else if (!strcmp(token->v, "not"))
-            { op=GAL_ARITHMETIC_OP_NOT;           nop=1;  }
+            { op=GAL_ARITHMETIC_OP_NOT;               nop=1;  }
           else if (!strcmp(token->v, "isblank"))
-            { op=GAL_ARITHMETIC_OP_ISBLANK;       nop=1;  }
+            { op=GAL_ARITHMETIC_OP_ISBLANK;           nop=1;  }
           else if (!strcmp(token->v, "where"))
-            { op=GAL_ARITHMETIC_OP_WHERE;         nop=3;  }
+            { op=GAL_ARITHMETIC_OP_WHERE;             nop=3;  }
 
           /* Bitwise operators. */
           else if (!strcmp(token->v, "bitand"))
-            { op=GAL_ARITHMETIC_OP_BITAND;        nop=2;  }
+            { op=GAL_ARITHMETIC_OP_BITAND;            nop=2;  }
           else if (!strcmp(token->v, "bitor"))
-            { op=GAL_ARITHMETIC_OP_BITOR;         nop=2;  }
+            { op=GAL_ARITHMETIC_OP_BITOR;             nop=2;  }
           else if (!strcmp(token->v, "bitxor"))
-            { op=GAL_ARITHMETIC_OP_BITXOR;        nop=2;  }
+            { op=GAL_ARITHMETIC_OP_BITXOR;            nop=2;  }
           else if (!strcmp(token->v, "lshift"))
-            { op=GAL_ARITHMETIC_OP_BITLSH;        nop=2;  }
+            { op=GAL_ARITHMETIC_OP_BITLSH;            nop=2;  }
           else if (!strcmp(token->v, "rshift"))
-            { op=GAL_ARITHMETIC_OP_BITRSH;        nop=2;  }
+            { op=GAL_ARITHMETIC_OP_BITRSH;            nop=2;  }
           else if (!strcmp(token->v, "bitnot"))
-            { op=GAL_ARITHMETIC_OP_BITNOT;        nop=1;  }
+            { op=GAL_ARITHMETIC_OP_BITNOT;            nop=1;  }
 
           /* Type conversion. */
           else if (!strcmp(token->v, "uint8"))
-            { op=GAL_ARITHMETIC_OP_TO_UINT8;      nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_UINT8;          nop=1;  }
           else if (!strcmp(token->v, "int8"))
-            { op=GAL_ARITHMETIC_OP_TO_INT8;       nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_INT8;           nop=1;  }
           else if (!strcmp(token->v, "uint16"))
-            { op=GAL_ARITHMETIC_OP_TO_UINT16;     nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_UINT16;         nop=1;  }
           else if (!strcmp(token->v, "int16"))
-            { op=GAL_ARITHMETIC_OP_TO_INT16;      nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_INT16;          nop=1;  }
           else if (!strcmp(token->v, "uint32"))
-            { op=GAL_ARITHMETIC_OP_TO_UINT32;     nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_UINT32;         nop=1;  }
           else if (!strcmp(token->v, "int32"))
-            { op=GAL_ARITHMETIC_OP_TO_INT32;      nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_INT32;          nop=1;  }
           else if (!strcmp(token->v, "uint64"))
-            { op=GAL_ARITHMETIC_OP_TO_UINT64;     nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_UINT64;         nop=1;  }
           else if (!strcmp(token->v, "int64"))
-            { op=GAL_ARITHMETIC_OP_TO_INT64;      nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_INT64;          nop=1;  }
           else if (!strcmp(token->v, "float32"))
-            { op=GAL_ARITHMETIC_OP_TO_FLOAT32;    nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_FLOAT32;        nop=1;  }
           else if (!strcmp(token->v, "float64"))
-            { op=GAL_ARITHMETIC_OP_TO_FLOAT64;    nop=1;  }
+            { op=GAL_ARITHMETIC_OP_TO_FLOAT64;        nop=1;  }
 
-          /* Filters. */
-          else if (!strcmp(token->v, "filter-median"))
-            { op=ARITHMETIC_OP_FILTER_MEDIAN;     nop=0;  }
+          /* Library wrappers. */
           else if (!strcmp(token->v, "filter-mean"))
-            { op=ARITHMETIC_OP_FILTER_MEAN;       nop=0;  }
+            { op=ARITHMETIC_OP_FILTER_MEAN;           nop=0;  }
+          else if (!strcmp(token->v, "filter-median"))
+            { op=ARITHMETIC_OP_FILTER_MEDIAN;         nop=0;  }
+          else if (!strcmp(token->v, "filter-sigclip-mean"))
+            { op=ARITHMETIC_OP_FILTER_SIGCLIP_MEAN;   nop=0;  }
+          else if (!strcmp(token->v, "filter-sigclip-median"))
+            { op=ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN; nop=0;  }
+          else if (!strcmp(token->v, "erode"))
+            { op=ARITHMETIC_OP_ERODE;                 nop=0;  }
+          else if (!strcmp(token->v, "dilate"))
+            { op=ARITHMETIC_OP_DILATE;                nop=0;  }
+          else if (!strcmp(token->v, "connected-components"))
+            { op=ARITHMETIC_OP_CONNECTED_COMPONENTS;  nop=0;  }
+          else if (!strcmp(token->v, "invert"))
+            { op=ARITHMETIC_OP_INVERT;                nop=0;  }
+          else if (!strcmp(token->v, "interpolate-medianngb"))
+            { op=ARITHMETIC_OP_INTERPOLATE_MEDIANNGB; nop=0;  }
+          else if (!strcmp(token->v, "collapse-sum"))
+            { op=ARITHMETIC_OP_COLLAPSE_SUM;          nop=0; }
+          else if (!strcmp(token->v, "collapse-mean"))
+            { op=ARITHMETIC_OP_COLLAPSE_MEAN;         nop=0; }
+          else if (!strcmp(token->v, "collapse-number"))
+            { op=ARITHMETIC_OP_COLLAPSE_NUMBER;       nop=0; }
 
 
           /* Finished checks with known operators */
           else
             error(EXIT_FAILURE, 0, "the argument \"%s\" could not be "
-                  "interpretted as a FITS file, number, or operator",
-                  token->v);
+                  "interpretted as a file name, named dataset, number, or "
+                  "operator", token->v);
 
 
           /* See if the arithmetic library must be called or not. */
@@ -658,9 +1037,34 @@ reversepolish(struct arithmeticparams *p)
                 {
                 case ARITHMETIC_OP_FILTER_MEAN:
                 case ARITHMETIC_OP_FILTER_MEDIAN:
+                case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
+                case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
                   wrapper_for_filter(p, token->v, op);
                   break;
 
+                case ARITHMETIC_OP_ERODE:
+                case ARITHMETIC_OP_DILATE:
+                  arithmetic_erode_dilate(p, token->v, op);
+                  break;
+
+                case ARITHMETIC_OP_CONNECTED_COMPONENTS:
+                  arithmetic_connected_components(p, token->v);
+                  break;
+
+                case ARITHMETIC_OP_INVERT:
+                  arithmetic_invert(p, token->v);
+                  break;
+
+                case ARITHMETIC_OP_INTERPOLATE_MEDIANNGB:
+                  arithmetic_interpolate(p, token->v);
+                  break;
+
+                case ARITHMETIC_OP_COLLAPSE_SUM:
+                case ARITHMETIC_OP_COLLAPSE_MEAN:
+                case ARITHMETIC_OP_COLLAPSE_NUMBER:
+                  arithmetic_collapse(p, token->v, op);
+                  break;
+
                 default:
                   error(EXIT_FAILURE, 0, "%s: a bug! please contact us at "
                         "%s to fix the problem. The code %d is not "
@@ -669,8 +1073,18 @@ reversepolish(struct arithmeticparams *p)
                 }
             }
         }
+
+      /* Increment the token counter. */
+      ++p->tokencounter;
     }
 
+
+  /* If there aren't any more operands (a variable has been set but not
+     used), then there is nothing to create. */
+  if(p->operands==NULL)
+    error(EXIT_FAILURE, 0, "no operands on the stack to write (as output)");
+
+
   /* If there is more than one node in the operands stack then the user has
      given too many operands which is an error. */
   if(p->operands->next!=NULL)
@@ -690,11 +1104,9 @@ reversepolish(struct arithmeticparams *p)
       filename=p->operands->filename;
       if( gal_fits_name_is_fits(filename) )
         {
-          p->operands->data=gal_fits_img_read(filename,hdu,p->cp.minmapsize,
-                                              0, 0);
-          p->refdata.wcs=p->operands->data->wcs;
-          p->refdata.nwcs=p->operands->data->nwcs;
-          p->operands->data->wcs=NULL;
+          p->operands->data=gal_array_read_one_ch(filename, hdu,
+                                                  p->cp.minmapsize);
+          p->refdata.wcs=gal_wcs_read(filename, hdu, 0, 0, &p->refdata.nwcs);
           if(!p->cp.quiet) printf(" - %s (hdu %s) is read.\n", filename, hdu);
         }
       else
@@ -732,6 +1144,7 @@ reversepolish(struct arithmeticparams *p)
      into `d1', so it is freed when freeing d1. */
   gal_data_free(d1);
   free(p->refdata.dsize);
+  gal_list_data_free(p->named);
 
   /* Clean up. Note that the tokens were taken from the command-line
      arguments, so the string within each token linked list must not be
@@ -762,7 +1175,7 @@ reversepolish(struct arithmeticparams *p)
 /*************             Top function            *************/
 /***************************************************************/
 void
-imgarith(struct arithmeticparams *p)
+arithmetic(struct arithmeticparams *p)
 {
   /* Parse the arguments */
   reversepolish(p);
