@@ -3,9 +3,9 @@ Arithmetic - Do arithmetic operations on images.
 Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,14 +31,24 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
    library. */
 enum arithmetic_prog_operators
 {
-  ARITHMETIC_OP_FILTER_MEDIAN=GAL_ARITHMETIC_OP_LAST_CODE,
+  ARITHMETIC_OP_FILTER_MEDIAN = GAL_ARITHMETIC_OP_LAST_CODE,
   ARITHMETIC_OP_FILTER_MEAN,
+  ARITHMETIC_OP_FILTER_SIGCLIP_MEAN,
+  ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN,
+  ARITHMETIC_OP_ERODE,
+  ARITHMETIC_OP_DILATE,
+  ARITHMETIC_OP_CONNECTED_COMPONENTS,
+  ARITHMETIC_OP_INVERT,
+  ARITHMETIC_OP_INTERPOLATE_MEDIANNGB,
+  ARITHMETIC_OP_COLLAPSE_SUM,
+  ARITHMETIC_OP_COLLAPSE_MEAN,
+  ARITHMETIC_OP_COLLAPSE_NUMBER,
 };
 
 
 
 
 void
-imgarith(struct arithmeticparams *p);
+arithmetic(struct arithmeticparams *p);
 
 #endif
