@@ -2,9 +2,9 @@
 Functions dealing with general aspects of all Gnuastro.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -34,27 +34,28 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Configuration macros: */
 
-#define GAL_CONFIG_VERSION              "0.4.108-509b"
+#define GAL_CONFIG_VERSION                  "0.6.54-6aa5"
 
-#define GAL_CONFIG_HAVE_LIBGIT2         1
+#define GAL_CONFIG_HAVE_LIBGIT2             1
 
-#define GAL_CONFIG_HAVE_WCSLIB_VERSION  1
+#define GAL_CONFIG_HAVE_FITS_IS_REENTRANT   @HAVE_FITS_IS_REENTRANT@
+#define GAL_CONFIG_HAVE_WCSLIB_VERSION      1
 
-#define GAL_CONFIG_HAVE_PTHREAD_BARRIER 1
+#define GAL_CONFIG_HAVE_PTHREAD_BARRIER     1
 
-#define GAL_CONFIG_BIN_OP_UINT8         1
-#define GAL_CONFIG_BIN_OP_INT8          0
-#define GAL_CONFIG_BIN_OP_UINT16        0
-#define GAL_CONFIG_BIN_OP_INT16         0
-#define GAL_CONFIG_BIN_OP_UINT32        0
-#define GAL_CONFIG_BIN_OP_INT32         0
-#define GAL_CONFIG_BIN_OP_UINT64        1
-#define GAL_CONFIG_BIN_OP_INT64         1
-#define GAL_CONFIG_BIN_OP_FLOAT32       1
-#define GAL_CONFIG_BIN_OP_FLOAT64       1
+#define GAL_CONFIG_BIN_OP_UINT8             @HAVE_BIN_OP_UINT8@
+#define GAL_CONFIG_BIN_OP_INT8              @HAVE_BIN_OP_INT8@
+#define GAL_CONFIG_BIN_OP_UINT16            @HAVE_BIN_OP_UINT16@
+#define GAL_CONFIG_BIN_OP_INT16             @HAVE_BIN_OP_INT16@
+#define GAL_CONFIG_BIN_OP_UINT32            @HAVE_BIN_OP_UINT32@
+#define GAL_CONFIG_BIN_OP_INT32             @HAVE_BIN_OP_INT32@
+#define GAL_CONFIG_BIN_OP_UINT64            @HAVE_BIN_OP_UINT64@
+#define GAL_CONFIG_BIN_OP_INT64             @HAVE_BIN_OP_INT64@
+#define GAL_CONFIG_BIN_OP_FLOAT32           @HAVE_BIN_OP_FLOAT32@
+#define GAL_CONFIG_BIN_OP_FLOAT64           @HAVE_BIN_OP_FLOAT64@
 
-#define GAL_CONFIG_SIZEOF_LONG          8
-#define GAL_CONFIG_SIZEOF_SIZE_T        8
+#define GAL_CONFIG_SIZEOF_LONG              8
+#define GAL_CONFIG_SIZEOF_SIZE_T            8
 
 
 /* C++ Preparations */
