@@ -3,9 +3,9 @@ multidim -- Functions for multi-dimensional operations.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Include other headers if necessary here. Note that other header files
    must be included before the C++ preparations below */
 #include <gnuastro/data.h>
+#include <gnuastro/blank.h>
 
 /* C++ Preparations */
 #undef __BEGIN_C_DECLS
@@ -52,6 +53,9 @@ __BEGIN_C_DECLS  /* From C++ preparations */
 size_t
 gal_dimension_total_size(size_t ndim, size_t *dsize);
 
+int
+gal_dimension_is_different(gal_data_t *first, gal_data_t *second);
+
 size_t *
 gal_dimension_increment(size_t ndim, size_t *dsize);
 
@@ -92,6 +96,23 @@ gal_dimension_dist_manhattan(size_t *a, size_t *b, size_t ndim);
 
 
 
+/************************************************************************/
+/********************    Collapsing a dimension    **********************/
+/************************************************************************/
+gal_data_t *
+gal_dimension_collapse_sum(gal_data_t *in, size_t c_dim, gal_data_t *weight);
+
+gal_data_t *
+gal_dimension_collapse_mean(gal_data_t *in, size_t c_dim,
+                            gal_data_t *weight);
+
+gal_data_t *
+gal_dimension_collapse_number(gal_data_t *in, size_t c_dim);
+
+
+
+
+
 /************************************************************************/
 /********************          Neighbors           **********************/
 /************************************************************************/
@@ -188,6 +209,12 @@ gal_dimension_dist_manhattan(size_t *a, size_t *b, size_t ndim);
     size_t nind, gdn_ind=index;                                         \
     uint8_t gdn_D, *gdn_is_start, *gdn_is_end, *gdn_is_edge, gdn_one=1; \
                                                                         \
+    /* A small sanity check. */                                         \
+    if(connectivity>ndim)                                               \
+      error(EXIT_FAILURE, 0, "%s: connectivity value (%d) is larger "   \
+            "than the number of dimensions (%zu)", __func__,            \
+            (int)connectivity, ndim);                                   \
+                                                                        \
     /* Initialize the start/end. */                                     \
     gdn_is_start=(uint8_t *)(&gdn_bitstr);                              \
     gdn_is_end=(uint8_t *)(&gdn_bitstr)+1;                              \
